#ifndef POKEPLATINUM_UNK_02098700_H
#define POKEPLATINUM_UNK_02098700_H

#include "struct_decls/struct_02098700_decl.h"

UnkStruct_02098700 * sub_02098700(int param0);
void sub_02098718(UnkStruct_02098700 * param0);
void sub_0209872C(UnkStruct_02098700 * param0, int param1, int param2);
void sub_020987BC(UnkStruct_02098700 * param0);
int sub_02098808(const UnkStruct_02098700 * param0, int param1);
int sub_02098828(const UnkStruct_02098700 * param0, int param1);
short sub_02098848(const UnkStruct_02098700 * param0, int param1);
short sub_02098868(const UnkStruct_02098700 * param0, int param1);
short sub_02098888(const UnkStruct_02098700 * param0, int param1);
short sub_020988A8(const UnkStruct_02098700 * param0, int param1);

#endif // POKEPLATINUM_UNK_02098700_H
